@@ -2,7 +2,7 @@
  * swigutil_pl.h :  utility functions and stuff for the SWIG Perl bindings
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -35,6 +35,7 @@
 #include "svn_delta.h"
 #include "svn_client.h"
 #include "svn_repos.h"
+#include "svn_private_config.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -50,53 +51,30 @@ extern "C" {
 
 
 
-#if SVN_SWIG_VERSION < 103024
-/* If this file is being included outside of a wrapper file, then need to
-   create stubs for some of the SWIG types. */
+#if defined(SVN_AVOID_CIRCULAR_LINKAGE_AT_ALL_COSTS_HACK)
+typedef apr_pool_t *(*svn_swig_pl_get_current_pool_t)(void);
+typedef void (*svn_swig_pl_set_current_pool_t)(apr_pool_t *pool);
 
-/* if SWIGEXPORT is defined, then we're in a wrapper. otherwise, we need
-   the prototypes and type definitions. */
-#ifndef SWIGEXPORT
-#define SVN_NEED_SWIG_TYPES
+void svn_swig_pl_bind_current_pool_fns(svn_swig_pl_get_current_pool_t get,
+                                       svn_swig_pl_set_current_pool_t set);
 #endif
 
-#ifdef SVN_NEED_SWIG_TYPES
-
-#if SVN_SWIG_VERSION >= 103020
-#include "perl5/precommon.swg"
-/* The following two aren't handled by the precommon.swg.  */
-#ifndef SWIG_MakePtr
-#define SWIG_MakePtr SWIG_Perl_MakePtr
-#endif
-#ifndef SWIG_ConvertPtr
-#define SWIG_ConvertPtr SWIG_Perl_ConvertPtr
-#endif
-#endif
-
-typedef struct _unnamed swig_type_info;
-
-swig_type_info *SWIG_TypeQuery(const char *name);
-int SWIG_ConvertPtr(SV *, void **, swig_type_info *, int flags);
-void SWIG_MakePtr(SV *, void *, swig_type_info *, int flags);
-
-#endif /* SVN_NEED_SWIG_TYPES */
-#endif /* SVN_SWIG_VERSION < 103024 */
-
-
-apr_pool_t *svn_swig_pl_make_pool (SV *obj);
+apr_pool_t *svn_swig_pl_make_pool(SV *obj);
 
 typedef enum perl_func_invoker {
     CALL_METHOD,
     CALL_SV
 } perl_func_invoker_t;
 
-svn_error_t *svn_swig_pl_callback_thunk (perl_func_invoker_t caller_func,
-		                         void *func,
-					 SV **result,
-					 const char *fmt, ...);
+svn_error_t *svn_swig_pl_callback_thunk(perl_func_invoker_t caller_func,
+                                        void *func,
+                                        SV **result,
+                                        const char *fmt, ...);
 
-SV *svn_swig_pl_prophash_to_hash (apr_hash_t *hash);
-SV *svn_swig_pl_convert_hash (apr_hash_t *hash, swig_type_info *tinfo);
+SV *svn_swig_pl_prophash_to_hash(apr_hash_t *hash);
+SV *svn_swig_pl_convert_hash(apr_hash_t *hash, swig_type_info *tinfo);
+
+SV *svn_swig_pl_convert_hash_of_revnum_t(apr_hash_t *hash);
 
 const apr_array_header_t *svn_swig_pl_strings_to_array(SV *source,
                                                        apr_pool_t *pool);
@@ -108,6 +86,8 @@ apr_hash_t *svn_swig_pl_objs_to_hash(SV *source, swig_type_info *tinfo,
 apr_hash_t *svn_swig_pl_objs_to_hash_by_name(SV *source,
                                              const char *typename,
                                              apr_pool_t *pool);
+apr_hash_t *svn_swig_pl_objs_to_hash_of_revnum_t(SV *source,
+                                                 apr_pool_t *pool);
 const apr_array_header_t *svn_swig_pl_objs_to_array(SV *source,
                                                     swig_type_info *tinfo,
                                                     apr_pool_t *pool);
@@ -127,6 +107,13 @@ svn_error_t * svn_swig_pl_thunk_log_receiver(void *py_receiver,
                                              const char *date,
                                              const char *msg,
                                              apr_pool_t *pool);
+
+/* thunked diff summarize callback.  */
+svn_error_t * svn_swig_pl_thunk_client_diff_summarize_func(
+                     const svn_client_diff_summarize_t *diff,
+                     void *baton,
+                     apr_pool_t *pool);
+
 /* thunked commit editor callback. */
 svn_error_t *svn_swig_pl_thunk_commit_callback(svn_revnum_t new_revision,
 					       const char *date,
@@ -140,11 +127,11 @@ svn_error_t *svn_swig_pl_thunk_history_func(void *baton,
                                             apr_pool_t *pool);
 
 /* thunked dir_delta authz read function. */
-svn_error_t *svn_swig_pl_thunk_authz_func (svn_boolean_t *allowed,
-                                           svn_fs_root_t *root,
-                                           const char *path,
-                                           void *baton,
-                                           apr_pool_t *pool);
+svn_error_t *svn_swig_pl_thunk_authz_func(svn_boolean_t *allowed,
+                                          svn_fs_root_t *root,
+                                          const char *path,
+                                          void *baton,
+                                          apr_pool_t *pool);
 
 /* ra callbacks. */
 svn_error_t *svn_ra_make_callbacks(svn_ra_callbacks_t **cb,
@@ -153,52 +140,52 @@ svn_error_t *svn_ra_make_callbacks(svn_ra_callbacks_t **cb,
 				   apr_pool_t *pool);
 
 /* thunked simple_prompt callback function */
-svn_error_t *svn_swig_pl_thunk_simple_prompt (svn_auth_cred_simple_t **cred,
-                                              void *baton,
-                                              const char *realm,
-                                              const char *username,
-                                              svn_boolean_t may_save,
-                                              apr_pool_t *pool);
+svn_error_t *svn_swig_pl_thunk_simple_prompt(svn_auth_cred_simple_t **cred,
+                                             void *baton,
+                                             const char *realm,
+                                             const char *username,
+                                             svn_boolean_t may_save,
+                                             apr_pool_t *pool);
 
 /* thunked username_prompt callback function */
-svn_error_t *svn_swig_pl_thunk_username_prompt (svn_auth_cred_username_t **cred,
-                                                void *baton,
-                                                const char *realm,
-                                                svn_boolean_t may_save,
-                                                apr_pool_t *pool);
+svn_error_t *svn_swig_pl_thunk_username_prompt(svn_auth_cred_username_t **cred,
+                                               void *baton,
+                                               const char *realm,
+                                               svn_boolean_t may_save,
+                                               apr_pool_t *pool);
 
 /* thunked ssl_server_trust_prompt callback function */
-svn_error_t *svn_swig_pl_thunk_ssl_server_trust_prompt(
-                              svn_auth_cred_ssl_server_trust_t **cred,
-                              void *baton,
-                              const char *realm,
-                              apr_uint32_t failures,
-                              const svn_auth_ssl_server_cert_info_t *cert_info,
-                              svn_boolean_t may_save,
-                              apr_pool_t *pool);
+svn_error_t *svn_swig_pl_thunk_ssl_server_trust_prompt
+  (svn_auth_cred_ssl_server_trust_t **cred,
+   void *baton,
+   const char *realm,
+   apr_uint32_t failures,
+   const svn_auth_ssl_server_cert_info_t *cert_info,
+   svn_boolean_t may_save,
+   apr_pool_t *pool);
 
 /* thunked ssl_client_cert callback function */
-svn_error_t *svn_swig_pl_thunk_ssl_client_cert_prompt (
-                                        svn_auth_cred_ssl_client_cert_t **cred,
-                                        void *baton,
-                                        const char *realm,
-                                        svn_boolean_t may_save,
-                                        apr_pool_t *pool);
+svn_error_t *svn_swig_pl_thunk_ssl_client_cert_prompt
+  (svn_auth_cred_ssl_client_cert_t **cred,
+   void *baton,
+   const char *realm,
+   svn_boolean_t may_save,
+   apr_pool_t *pool);
 
 /* thunked ssl_client_cert_pw callback function */
-svn_error_t *svn_swig_pl_thunk_ssl_client_cert_pw_prompt (
-                                     svn_auth_cred_ssl_client_cert_pw_t **cred,
-                                     void *baton,
-                                     const char *realm,
-                                     svn_boolean_t may_save,
-                                     apr_pool_t *pool);
+svn_error_t *svn_swig_pl_thunk_ssl_client_cert_pw_prompt
+  (svn_auth_cred_ssl_client_cert_pw_t **cred,
+   void *baton,
+   const char *realm,
+   svn_boolean_t may_save,
+   apr_pool_t *pool);
 
 /* thunked callback for svn_ra_get_wc_prop_func_t */
-svn_error_t *thunk_get_wc_prop (void *baton,
-                                const char *relpath,
-                                const char *name,
-                                const svn_string_t **value,
-                                apr_pool_t *pool);
+svn_error_t *thunk_get_wc_prop(void *baton,
+                               const char *relpath,
+                               const char *name,
+                               const svn_string_t **value,
+                               apr_pool_t *pool);
 
 /* Thunked version of svn_wc_notify_func_t callback type */
 void svn_swig_pl_notify_func(void * baton,
@@ -211,13 +198,20 @@ void svn_swig_pl_notify_func(void * baton,
 			     svn_revnum_t revision);
 
 
-/* Thunked version of svn_client_get_commit_log_t callback type. */
+/* Thunked version of svn_client_get_commit_log3_t callback type. */
 svn_error_t *svn_swig_pl_get_commit_log_func(const char **log_msg,
                                              const char **tmp_file,
-                                             apr_array_header_t *commit_items,
+                                             const apr_array_header_t *
+                                             commit_items,
                                              void *baton,
                                              apr_pool_t *pool);
 
+/* Thunked version of svn_client_info_t callback type. */
+svn_error_t *svn_swig_pl_info_receiver(void *baton,
+                                       const char *path,
+                                       const svn_info_t *info,
+                                       apr_pool_t *pool);
+
 /* Thunked version of svn_wc_cancel_func_t callback type. */
 svn_error_t *svn_swig_pl_cancel_func(void *cancel_baton);
 
@@ -226,16 +220,16 @@ void svn_swig_pl_status_func(void *baton,
                              const char *path,
                              svn_wc_status_t *status);
 /* Thunked version of svn_client_blame_receiver_t callback type. */
-svn_error_t *svn_swig_pl_blame_func (void *baton,
-                                     apr_int64_t line_no,
-                                     svn_revnum_t revision,
-                                     const char *author,
-                                     const char *date,
-                                     const char *line,
-                                     apr_pool_t *pool);
+svn_error_t *svn_swig_pl_blame_func(void *baton,
+                                    apr_int64_t line_no,
+                                    svn_revnum_t revision,
+                                    const char *author,
+                                    const char *date,
+                                    const char *line,
+                                    apr_pool_t *pool);
 
 /* Thunked config enumerator */
-svn_boolean_t svn_swig_pl_thunk_config_enumerator (const char *name, const char *value, void *baton);
+svn_boolean_t svn_swig_pl_thunk_config_enumerator(const char *name, const char *value, void *baton);
 
 /* helper for making the editor */
 void svn_delta_make_editor(svn_delta_editor_t **editor,
@@ -243,14 +237,29 @@ void svn_delta_make_editor(svn_delta_editor_t **editor,
                            SV *perl_editor,
                            apr_pool_t *pool);
 
+void svn_delta_wrap_window_handler(svn_txdelta_window_handler_t *handler,
+                                   void **h_baton,
+                                   SV *callback,
+                                   apr_pool_t *pool);
+
 /* svn_stream_t helpers */
-svn_error_t *svn_swig_pl_make_stream (svn_stream_t **stream, SV *obj);
-SV *svn_swig_pl_from_stream (svn_stream_t *stream);
+svn_error_t *svn_swig_pl_make_stream(svn_stream_t **stream, SV *obj);
+SV *svn_swig_pl_from_stream(svn_stream_t *stream);
 
 /* apr_file_t * */
-apr_file_t *svn_swig_pl_make_file (SV *file, apr_pool_t *pool);
+apr_file_t *svn_swig_pl_make_file(SV *file, apr_pool_t *pool);
 
-void svn_swig_pl_hold_ref_in_pool (apr_pool_t *pool, SV *sv);
+void svn_swig_pl_hold_ref_in_pool(apr_pool_t *pool, SV *sv);
+
+/* md5 access class */
+SV *svn_swig_pl_from_md5(unsigned char *digest);
+
+svn_error_t *svn_swig_pl_ra_lock_callback(void *baton,
+                                          const char *path,
+                                          svn_boolean_t do_lock,
+                                          const svn_lock_t *lock,
+                                          svn_error_t *ra_err,
+                                          apr_pool_t *pool);
 
 #ifdef __cplusplus
 }
