@@ -2,7 +2,7 @@
  * opt.c :  option and argument parsing for Subversion command lines
  *
  * ====================================================================
- * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2009 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -36,7 +36,11 @@
 #include "svn_path.h"
 #include "svn_utf.h"
 #include "svn_time.h"
+#include "svn_props.h"
 
+#include "private/svn_opt_private.h"
+
+#include "opt.h"
 #include "svn_private_config.h"
 
 
@@ -66,32 +70,6 @@ svn_opt_get_canonical_subcommand2(const svn_opt_subcommand_desc2_t *table,
   return NULL;
 }
 
-
-const svn_opt_subcommand_desc_t *
-svn_opt_get_canonical_subcommand(const svn_opt_subcommand_desc_t *table,
-                                 const char *cmd_name)
-{
-  int i = 0;
-
-  if (cmd_name == NULL)
-    return NULL;
-
-  while (table[i].name) {
-    int j;
-    if (strcmp(cmd_name, table[i].name) == 0)
-      return table + i;
-    for (j = 0; (j < SVN_OPT_MAX_ALIASES) && table[i].aliases[j]; j++)
-      if (strcmp(cmd_name, table[i].aliases[j]) == 0)
-        return table + i;
-
-    i++;
-  }
-
-  /* If we get here, there was no matching subcommand name or alias. */
-  return NULL;
-}
-
-
 const apr_getopt_option_t *
 svn_opt_get_option_from_code2(int code,
                               const apr_getopt_option_t *option_table,
@@ -260,89 +238,12 @@ print_command_info2(const svn_opt_subcommand_desc2_t *cmd,
 
           for (i = 0; global_options[i]; i++)
             {
-              
+
               /* convert each option code into an option */
               option =
                 svn_opt_get_option_from_code2(global_options[i],
                                               options_table,
                                               cmd, pool);
-              
-              /* print the option's docstring */
-              if (option && option->description)
-                {
-                  const char *optstr;
-                  svn_opt_format_option(&optstr, option, TRUE, pool);
-                  SVN_ERR(svn_cmdline_fprintf(stream, pool, "  %s\n",
-                                              optstr));
-                }
-            }
-        }
-          
-      if (have_options)
-        SVN_ERR(svn_cmdline_fprintf(stream, pool, "\n"));
-    }
-
-  return SVN_NO_ERROR;
-}
-
-
-/* Same as print_command_info2(), but with deprecated struct revision. */
-static svn_error_t *
-print_command_info(const svn_opt_subcommand_desc_t *cmd,
-                   const apr_getopt_option_t *options_table,
-                   svn_boolean_t help,
-                   apr_pool_t *pool,
-                   FILE *stream)
-{
-  svn_boolean_t first_time;
-  apr_size_t i;
-
-  /* Print the canonical command name. */
-  SVN_ERR(svn_cmdline_fputs(cmd->name, stream, pool));
-
-  /* Print the list of aliases. */
-  first_time = TRUE;
-  for (i = 0; i < SVN_OPT_MAX_ALIASES; i++)
-    {
-      if (cmd->aliases[i] == NULL)
-        break;
-
-      if (first_time) {
-        SVN_ERR(svn_cmdline_fputs(" (", stream, pool));
-        first_time = FALSE;
-      }
-      else
-        SVN_ERR(svn_cmdline_fputs(", ", stream, pool));
-
-      SVN_ERR(svn_cmdline_fputs(cmd->aliases[i], stream, pool));
-    }
-
-  if (! first_time)
-    SVN_ERR(svn_cmdline_fputs(")", stream, pool));
-
-  if (help)
-    {
-      const apr_getopt_option_t *option;
-      svn_boolean_t have_options = FALSE;
-
-      SVN_ERR(svn_cmdline_fprintf(stream, pool, ": %s", _(cmd->help)));
-
-      /* Loop over all valid option codes attached to the subcommand */
-      for (i = 0; i < SVN_OPT_MAX_OPTIONS; i++)
-        {
-          if (cmd->valid_options[i])
-            {
-              if (have_options == FALSE)
-                {
-                  SVN_ERR(svn_cmdline_fputs(_("\nValid options:\n"),
-                                            stream, pool));
-                  have_options = TRUE;
-                }
-
-              /* convert each option code into an option */
-              option =
-                svn_opt_get_option_from_code(cmd->valid_options[i],
-                                             options_table);
 
               /* print the option's docstring */
               if (option && option->description)
@@ -362,7 +263,6 @@ print_command_info(const svn_opt_subcommand_desc_t *cmd,
   return SVN_NO_ERROR;
 }
 
-
 void
 svn_opt_print_generic_help2(const char *header,
                             const svn_opt_subcommand_desc2_t *cmd_table,
@@ -402,46 +302,6 @@ svn_opt_print_generic_help2(const char *header,
   svn_error_clear(err);
 }
 
-
-void
-svn_opt_print_generic_help(const char *header,
-                           const svn_opt_subcommand_desc_t *cmd_table,
-                           const apr_getopt_option_t *opt_table,
-                           const char *footer,
-                           apr_pool_t *pool, FILE *stream)
-{
-  int i = 0;
-  svn_error_t *err;
-
-  if (header)
-    if ((err = svn_cmdline_fputs(header, stream, pool)))
-      goto print_error;
-
-  while (cmd_table[i].name)
-    {
-      if ((err = svn_cmdline_fputs("   ", stream, pool))
-          || (err = print_command_info(cmd_table + i, opt_table, FALSE,
-                                       pool, stream))
-          || (err = svn_cmdline_fputs("\n", stream, pool)))
-        goto print_error;
-      i++;
-    }
-
-  if ((err = svn_cmdline_fputs("\n", stream, pool)))
-    goto print_error;
-
-  if (footer)
-    if ((err = svn_cmdline_fputs(footer, stream, pool)))
-      goto print_error;
-
-  return;
-
- print_error:
-  svn_handle_error2(err, stderr, FALSE, "svn: ");
-  svn_error_clear(err);
-}
-
-
 void
 svn_opt_format_option(const char **string,
                       const apr_getopt_option_t *opt,
@@ -497,39 +357,6 @@ svn_opt_subcommand_help3(const char *subcommand,
   }
 }
 
-void
-svn_opt_subcommand_help2(const char *subcommand,
-                         const svn_opt_subcommand_desc2_t *table,
-                         const apr_getopt_option_t *options_table,
-                         apr_pool_t *pool)
-{
-  svn_opt_subcommand_help3(subcommand, table, options_table,
-                           NULL, pool);
-}
-
-
-void
-svn_opt_subcommand_help(const char *subcommand,
-                        const svn_opt_subcommand_desc_t *table,
-                        const apr_getopt_option_t *options_table,
-                        apr_pool_t *pool)
-{
-  const svn_opt_subcommand_desc_t *cmd =
-    svn_opt_get_canonical_subcommand(table, subcommand);
-  svn_error_t *err;
-
-  if (cmd)
-    err = print_command_info(cmd, options_table, TRUE, pool, stdout);
-  else
-    err = svn_cmdline_fprintf(stderr, pool,
-                              _("\"%s\": unknown command.\n\n"), subcommand);
-
-  if (err) {
-    svn_handle_error2(err, stderr, FALSE, "svn: ");
-    svn_error_clear(err);
-  }
-}
-
 
 
 /*** Parsing revision and date options. ***/
@@ -808,99 +635,73 @@ svn_opt_parse_path(svn_opt_revision_t *rev,
                    const char *path /* UTF-8! */,
                    apr_pool_t *pool)
 {
-  int i;
+  const char *peg_rev;
 
-  /* scanning from right to left, just to be friendly to any
-     screwed-up filenames that might *actually* contain @-signs.  :-) */
-  for (i = (strlen(path) - 1); i >= 0; i--)
+  SVN_ERR(svn_opt__split_arg_at_peg_revision(truepath, &peg_rev, path, pool));
+
+  /* Parse the peg revision, if one was found */
+  if (strlen(peg_rev))
     {
-      /* If we hit a path separator, stop looking. */
-      /* This is OK only because our revision specifiers can't contain '/'. */
-      if (path[i] == '/')
-        break;
+      int ret;
+      svn_opt_revision_t start_revision, end_revision;
 
-      if (path[i] == '@')
+      end_revision.kind = svn_opt_revision_unspecified;
+
+      if (peg_rev[1] == '\0')  /* looking at empty peg revision */
         {
-          int ret;
-          svn_opt_revision_t start_revision, end_revision;
-
-          end_revision.kind = svn_opt_revision_unspecified;
-
-          if (path[i + 1] == '\0')  /* looking at empty peg revision */
-            {
-              ret = 0;
-              start_revision.kind = svn_opt_revision_unspecified;
-            }
-          else  /* looking at non-empty peg revision */
-            {
-              const char *rev_str = path + i + 1;
-
-              /* URLs get treated differently from wc paths. */
-              if (svn_path_is_url(path))
-                {
-                  /* URLs are URI-encoded, so we look for dates with
-                     URI-encoded delimeters.  */
-                  int rev_len = strlen(rev_str);
-                  if (rev_len > 6
-                      && rev_str[0] == '%'
-                      && rev_str[1] == '7'
-                      && (rev_str[2] == 'B'
-                          || rev_str[2] == 'b')
-                      && rev_str[rev_len-3] == '%'
-                      && rev_str[rev_len-2] == '7'
-                      && (rev_str[rev_len-1] == 'D'
-                          || rev_str[rev_len-1] == 'd'))
-                    {
-                      rev_str = svn_path_uri_decode(rev_str, pool);
-                    }
-                }
-              ret = svn_opt_parse_revision(&start_revision,
-                                           &end_revision,
-                                           rev_str, pool);
-            }
-
-          if (ret || end_revision.kind != svn_opt_revision_unspecified)
-            return svn_error_createf(SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
-                                     _("Syntax error parsing revision '%s'"),
-                                     path + i + 1);
-
-          *truepath = apr_pstrmemdup(pool, path, i);
-          rev->kind = start_revision.kind;
-          rev->value = start_revision.value;
-
-          return SVN_NO_ERROR;
+          ret = 0;
+          start_revision.kind = svn_opt_revision_unspecified;
         }
-    }
+      else  /* looking at non-empty peg revision */
+        {
+          const char *rev_str = &peg_rev[1];
 
-  /* Didn't find an @-sign. */
-  *truepath = path;
-  rev->kind = svn_opt_revision_unspecified;
+          /* URLs get treated differently from wc paths. */
+          if (svn_path_is_url(path))
+            {
+              /* URLs are URI-encoded, so we look for dates with
+                 URI-encoded delimeters.  */
+              int rev_len = strlen(rev_str);
+              if (rev_len > 6
+                  && rev_str[0] == '%'
+                  && rev_str[1] == '7'
+                  && (rev_str[2] == 'B'
+                      || rev_str[2] == 'b')
+                  && rev_str[rev_len-3] == '%'
+                  && rev_str[rev_len-2] == '7'
+                  && (rev_str[rev_len-1] == 'D'
+                      || rev_str[rev_len-1] == 'd'))
+                {
+                  rev_str = svn_path_uri_decode(rev_str, pool);
+                }
+            }
+          ret = svn_opt_parse_revision(&start_revision,
+                                       &end_revision,
+                                       rev_str, pool);
+        }
+
+      if (ret || end_revision.kind != svn_opt_revision_unspecified)
+        return svn_error_createf(SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
+                                 _("Syntax error parsing revision '%s'"),
+                                 &peg_rev[1]);
+
+      rev->kind = start_revision.kind;
+      rev->value = start_revision.value;
+    }
+  else
+    {
+      /* Didn't find a peg revision. */
+      rev->kind = svn_opt_revision_unspecified;
+    }
 
   return SVN_NO_ERROR;
 }
 
 
+/* Note: This is substantially copied into svn_client_args_to_target_array() in
+ * order to move to libsvn_client while maintaining backward compatibility. */
 svn_error_t *
-svn_opt_args_to_target_array2(apr_array_header_t **targets_p,
-                              apr_getopt_t *os,
-                              apr_array_header_t *known_targets,
-                              apr_pool_t *pool)
-{
-  svn_error_t *err = svn_opt_args_to_target_array3(targets_p, os, 
-                                                   known_targets, pool);
-
-  if (err && err->apr_err == SVN_ERR_RESERVED_FILENAME_SPECIFIED)
-    {
-      svn_error_clear(err);
-      return SVN_NO_ERROR;
-    }
-
-  return err;
-}
-
-
-svn_error_t *
-svn_opt_args_to_target_array3(apr_array_header_t **targets_p,
+svn_opt__args_to_target_array(apr_array_header_t **targets_p,
                               apr_getopt_t *os,
                               apr_array_header_t *known_targets,
                               apr_pool_t *pool)
@@ -942,99 +743,42 @@ svn_opt_args_to_target_array3(apr_array_header_t **targets_p,
   for (i = 0; i < input_targets->nelts; i++)
     {
       const char *utf8_target = APR_ARRAY_IDX(input_targets, i, const char *);
-      const char *peg_start = NULL; /* pointer to the peg revision, if any */
+      const char *true_target;
       const char *target;      /* after all processing is finished */
-      int j;
+      const char *peg_rev;
 
-      /* Remove a peg revision, if any, in the target so that it can
-         be properly canonicalized, otherwise the canonicalization
-         does not treat a ".@BASE" as a "." with a BASE peg revision,
-         and it is not canonicalized to "@BASE".  If any peg revision
-         exists, it is appended to the final canonicalized path or
-         URL.  Do not use svn_opt_parse_path() because the resulting
-         peg revision is a structure that would have to be converted
-         back into a string.  Converting from a string date to the
-         apr_time_t field in the svn_opt_revision_value_t and back to
-         a string would not necessarily preserve the exact bytes of
-         the input date, so its easier just to keep it in string
-         form. */
-      for (j = (strlen(utf8_target) - 1); j >= 0; --j)
-        {
-          /* If we hit a path separator, stop looking.  This is OK
-              only because our revision specifiers can't contain
-              '/'. */
-          if (utf8_target[j] == '/')
-            break;
-          if (utf8_target[j] == '@')
-            {
-              peg_start = utf8_target + j;
-              break;
-            }
-        }
-      if (peg_start)
-        utf8_target = apr_pstrmemdup(pool,
-                                     utf8_target,
-                                     peg_start - utf8_target);
+      /*
+       * This is needed so that the target can be properly canonicalized,
+       * otherwise the canonicalization does not treat a ".@BASE" as a "."
+       * with a BASE peg revision, and it is not canonicalized to "@BASE".
+       * If any peg revision exists, it is appended to the final
+       * canonicalized path or URL.  Do not use svn_opt_parse_path()
+       * because the resulting peg revision is a structure that would have
+       * to be converted back into a string.  Converting from a string date
+       * to the apr_time_t field in the svn_opt_revision_value_t and back to
+       * a string would not necessarily preserve the exact bytes of the
+       * input date, so its easier just to keep it in string form.
+       */
+      SVN_ERR(svn_opt__split_arg_at_peg_revision(&true_target, &peg_rev,
+                                                 utf8_target, pool));
 
       /* URLs and wc-paths get treated differently. */
-      if (svn_path_is_url(utf8_target))
+      if (svn_path_is_url(true_target))
         {
-          /* No need to canonicalize a URL's case or path separators. */
-
-          /* Convert to URI. */
-          target = svn_path_uri_from_iri(utf8_target, pool);
-          /* Auto-escape some ASCII characters. */
-          target = svn_path_uri_autoescape(target, pool);
-
-          /* The above doesn't guarantee a valid URI. */
-          if (! svn_path_is_uri_safe(target))
-            return svn_error_createf(SVN_ERR_BAD_URL, 0,
-                                     _("URL '%s' is not properly URI-encoded"),
-                                     utf8_target);
-
-          /* Verify that no backpaths are present in the URL. */
-          if (svn_path_is_backpath_present(target))
-            return svn_error_createf(SVN_ERR_BAD_URL, 0,
-                                     _("URL '%s' contains a '..' element"),
-                                     utf8_target);
-
-          /* strip any trailing '/' */
-          target = svn_path_canonicalize(target, pool);
+          SVN_ERR(svn_opt__arg_canonicalize_url(&true_target, true_target,
+                                                 pool));
         }
       else  /* not a url, so treat as a path */
         {
-          const char *apr_target;
           const char *base_name;
-          char *truenamed_target; /* APR-encoded */
-          apr_status_t apr_err;
 
-          /* canonicalize case, and change all separators to '/'. */
-          SVN_ERR(svn_path_cstring_from_utf8(&apr_target, utf8_target,
-                                             pool));
-          apr_err = apr_filepath_merge(&truenamed_target, "", apr_target,
-                                       APR_FILEPATH_TRUENAME, pool);
-
-          if (!apr_err)
-            /* We have a canonicalized APR-encoded target now. */
-            apr_target = truenamed_target;
-          else if (APR_STATUS_IS_ENOENT(apr_err))
-            /* It's okay for the file to not exist, that just means we
-               have to accept the case given to the client. We'll use
-               the original APR-encoded target. */
-            ;
-          else
-            return svn_error_createf(apr_err, NULL,
-                                     _("Error resolving case of '%s'"),
-                                     svn_path_local_style(utf8_target,
-                                                          pool));
-
-          /* convert back to UTF-8. */
-          SVN_ERR(svn_path_cstring_to_utf8(&target, apr_target, pool));
-          target = svn_path_canonicalize(target, pool);
+          SVN_ERR(svn_opt__arg_canonicalize_path(&true_target, true_target,
+                                                 pool));
 
           /* If the target has the same name as a Subversion
              working copy administrative dir, skip it. */
-          base_name = svn_path_basename(target, pool);
+          base_name = svn_path_basename(true_target, pool);
+
           /* FIXME:
              The canonical list of administrative directory names is
              maintained in libsvn_wc/adm_files.c:svn_wc_set_adm_dir().
@@ -1047,15 +791,12 @@ svn_opt_args_to_target_array3(apr_array_header_t **targets_p,
             {
               err = svn_error_createf(SVN_ERR_RESERVED_FILENAME_SPECIFIED,
                                       err, _("'%s' ends in a reserved name"),
-                                      target);
+                                      utf8_target);
               continue;
             }
         }
 
-      /* Append the peg revision back to the canonicalized target if
-         there was a peg revision. */
-      if (peg_start)
-        target = apr_pstrcat(pool, target, peg_start, NULL);
+      target = apr_pstrcat(pool, true_target, peg_rev, NULL);
 
       APR_ARRAY_PUSH(output_targets, const char *) = target;
     }
@@ -1069,78 +810,156 @@ svn_opt_args_to_target_array3(apr_array_header_t **targets_p,
   return err;
 }
 
-
 svn_error_t *
-svn_opt_args_to_target_array(apr_array_header_t **targets_p,
-                             apr_getopt_t *os,
-                             apr_array_header_t *known_targets,
-                             svn_opt_revision_t *start_revision,
-                             svn_opt_revision_t *end_revision,
-                             svn_boolean_t extract_revisions,
-                             apr_pool_t *pool)
+svn_opt_parse_revprop(apr_hash_t **revprop_table_p, const char *revprop_spec,
+                      apr_pool_t *pool)
 {
-  apr_array_header_t *output_targets;
+  const char *sep, *propname;
+  svn_string_t *propval;
 
-  SVN_ERR(svn_opt_args_to_target_array2(&output_targets, os,
-                                        known_targets, pool));
+  if (! *revprop_spec)
+    return svn_error_create(SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
+                            _("Revision property pair is empty"));
 
-  if (extract_revisions)
+  if (! *revprop_table_p)
+    *revprop_table_p = apr_hash_make(pool);
+
+  sep = strchr(revprop_spec, '=');
+  if (sep)
     {
-      svn_opt_revision_t temprev;
-      const char *path;
-
-      if (output_targets->nelts > 0)
-        {
-          path = APR_ARRAY_IDX(output_targets, 0, const char *);
-          SVN_ERR(svn_opt_parse_path(&temprev, &path, path, pool));
-          if (temprev.kind != svn_opt_revision_unspecified)
-            {
-              APR_ARRAY_IDX(output_targets, 0, const char *) = path;
-              start_revision->kind = temprev.kind;
-              start_revision->value = temprev.value;
-            }
-        }
-      if (output_targets->nelts > 1)
-        {
-          path = APR_ARRAY_IDX(output_targets, 1, const char *);
-          SVN_ERR(svn_opt_parse_path(&temprev, &path, path, pool));
-          if (temprev.kind != svn_opt_revision_unspecified)
-            {
-              APR_ARRAY_IDX(output_targets, 1, const char *) = path;
-              end_revision->kind = temprev.kind;
-              end_revision->value = temprev.value;
-            }
-        }
+      propname = apr_pstrndup(pool, revprop_spec, sep - revprop_spec);
+      SVN_ERR(svn_utf_cstring_to_utf8(&propname, propname, pool));
+      propval = svn_string_create(sep + 1, pool);
+    }
+  else
+    {
+      SVN_ERR(svn_utf_cstring_to_utf8(&propname, revprop_spec, pool));
+      propval = svn_string_create("", pool);
     }
 
-  *targets_p = output_targets;
+  if (!svn_prop_name_is_valid(propname))
+    return svn_error_createf(SVN_ERR_CLIENT_PROPERTY_NAME, NULL,
+                             _("'%s' is not a valid Subversion property name"),
+                             propname);
+
+  apr_hash_set(*revprop_table_p, propname, APR_HASH_KEY_STRING, propval);
+
   return SVN_NO_ERROR;
 }
 
+svn_error_t *
+svn_opt__split_arg_at_peg_revision(const char **true_target,
+                                   const char **peg_revision,
+                                   const char *utf8_target,
+                                   apr_pool_t *pool)
+{
+  const char *peg_start = NULL; /* pointer to the peg revision, if any */
+  int j;
 
-/* Print version info for PGM_NAME.  If QUIET is  true, print in
- * brief.  Else if QUIET is not true, print the version more
- * verbosely, and if FOOTER is non-null, print it following the
- * version information.
- *
- * Use POOL for temporary allocations.
- */
-static svn_error_t *
-print_version_info(const char *pgm_name,
-                   const char *footer,
-                   svn_boolean_t quiet,
-                   apr_pool_t *pool)
+  for (j = (strlen(utf8_target) - 1); j >= 0; --j)
+    {
+      /* If we hit a path separator, stop looking.  This is OK
+          only because our revision specifiers can't contain '/'. */
+      if (utf8_target[j] == '/')
+        break;
+
+      if (utf8_target[j] == '@')
+        {
+          peg_start = &utf8_target[j];
+          break;
+        }
+    }
+
+  if (peg_start)
+    {
+      *true_target = apr_pstrmemdup(pool, utf8_target, j);
+      *peg_revision = apr_pstrdup(pool, peg_start);
+    }
+  else
+    {
+      *true_target = utf8_target;
+      *peg_revision = "";
+    }
+
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *
+svn_opt__arg_canonicalize_url(const char **url_out, const char *url_in,
+                              apr_pool_t *pool)
+{
+  const char *target;
+
+  /* Convert to URI. */
+  target = svn_path_uri_from_iri(url_in, pool);
+  /* Auto-escape some ASCII characters. */
+  target = svn_path_uri_autoescape(target, pool);
+
+  /* The above doesn't guarantee a valid URI. */
+  if (! svn_path_is_uri_safe(target))
+    return svn_error_createf(SVN_ERR_BAD_URL, 0,
+                             _("URL '%s' is not properly URI-encoded"),
+                             target);
+
+  /* Verify that no backpaths are present in the URL. */
+  if (svn_path_is_backpath_present(target))
+    return svn_error_createf(SVN_ERR_BAD_URL, 0,
+                             _("URL '%s' contains a '..' element"),
+                             target);
+
+  /* strip any trailing '/' and collapse other redundant elements */
+  target = svn_path_canonicalize(target, pool);
+
+  *url_out = target;
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *
+svn_opt__arg_canonicalize_path(const char **path_out, const char *path_in,
+                               apr_pool_t *pool)
+{
+  const char *apr_target;
+  char *truenamed_target; /* APR-encoded */
+  apr_status_t apr_err;
+
+  /* canonicalize case, and change all separators to '/'. */
+  SVN_ERR(svn_path_cstring_from_utf8(&apr_target, path_in, pool));
+  apr_err = apr_filepath_merge(&truenamed_target, "", apr_target,
+                               APR_FILEPATH_TRUENAME, pool);
+
+  if (!apr_err)
+    /* We have a canonicalized APR-encoded target now. */
+    apr_target = truenamed_target;
+  else if (APR_STATUS_IS_ENOENT(apr_err))
+    /* It's okay for the file to not exist, that just means we
+       have to accept the case given to the client. We'll use
+       the original APR-encoded target. */
+    ;
+  else
+    return svn_error_createf(apr_err, NULL,
+                             _("Error resolving case of '%s'"),
+                             svn_path_local_style(path_in, pool));
+
+  /* convert back to UTF-8. */
+  SVN_ERR(svn_path_cstring_to_utf8(path_out, apr_target, pool));
+  *path_out = svn_path_canonicalize(*path_out, pool);
+
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *
+svn_opt__print_version_info(const char *pgm_name,
+                            const char *footer,
+                            svn_boolean_t quiet,
+                            apr_pool_t *pool)
 {
   if (quiet)
-    {
-      SVN_ERR(svn_cmdline_printf(pool, "%s\n", SVN_VER_NUMBER));
-      return SVN_NO_ERROR;
-    }
+    return svn_cmdline_printf(pool, "%s\n", SVN_VER_NUMBER);
 
   SVN_ERR(svn_cmdline_printf(pool, _("%s, version %s\n"
                                      "   compiled %s, %s\n\n"), pgm_name,
                              SVN_VERSION, __DATE__, __TIME__));
-  SVN_ERR(svn_cmdline_fputs(_("Copyright (C) 2000-2008 CollabNet.\n"
+  SVN_ERR(svn_cmdline_fputs(_("Copyright (C) 2000-2009 CollabNet.\n"
                               "Subversion is open source software, see"
                               " http://subversion.tigris.org/\n"
                               "This product includes software developed by "
@@ -1183,7 +1002,8 @@ svn_opt_print_help3(apr_getopt_t *os,
                                  global_options, pool);
       }
   else if (print_version)   /* just --version */
-    SVN_ERR(print_version_info(pgm_name, version_footer, quiet, pool));
+    SVN_ERR(svn_opt__print_version_info(pgm_name, version_footer, quiet,
+                                        pool));
   else if (os && !targets->nelts)            /* `-h', `--help', or `help' */
     svn_opt_print_generic_help2(header,
                                 cmd_table,
@@ -1197,70 +1017,3 @@ svn_opt_print_help3(apr_getopt_t *os,
 
   return SVN_NO_ERROR;
 }
-
-
-svn_error_t *
-svn_opt_print_help2(apr_getopt_t *os,
-                    const char *pgm_name,
-                    svn_boolean_t print_version,
-                    svn_boolean_t quiet,
-                    const char *version_footer,
-                    const char *header,
-                    const svn_opt_subcommand_desc2_t *cmd_table,
-                    const apr_getopt_option_t *option_table,
-                    const char *footer,
-                    apr_pool_t *pool)
-{
-  return svn_opt_print_help3(os,
-                             pgm_name,
-                             print_version,
-                             quiet,
-                             version_footer,
-                             header,
-                             cmd_table,
-                             option_table,
-                             NULL,
-                             footer,
-                             pool);
-}
-
-
-svn_error_t *
-svn_opt_print_help(apr_getopt_t *os,
-                   const char *pgm_name,
-                   svn_boolean_t print_version,
-                   svn_boolean_t quiet,
-                   const char *version_footer,
-                   const char *header,
-                   const svn_opt_subcommand_desc_t *cmd_table,
-                   const apr_getopt_option_t *option_table,
-                   const char *footer,
-                   apr_pool_t *pool)
-{
-  apr_array_header_t *targets = NULL;
-  int i;
-
-  if (os)
-    SVN_ERR(svn_opt_parse_all_args(&targets, os, pool));
-
-  if (os && targets->nelts)  /* help on subcommand(s) requested */
-    for (i = 0; i < targets->nelts; i++)
-      {
-        svn_opt_subcommand_help(APR_ARRAY_IDX(targets, i, const char *),
-                                cmd_table, option_table, pool);
-      }
-  else if (print_version)   /* just --version */
-    SVN_ERR(print_version_info(pgm_name, version_footer, quiet, pool));
-  else if (os && !targets->nelts)            /* `-h', `--help', or `help' */
-    svn_opt_print_generic_help(header,
-                               cmd_table,
-                               option_table,
-                               footer,
-                               pool,
-                               stdout);
-  else                                       /* unknown option or cmd */
-    SVN_ERR(svn_cmdline_fprintf(stderr, pool,
-                                _("Type '%s help' for usage.\n"), pgm_name));
-
-  return SVN_NO_ERROR;
-}
