@@ -44,8 +44,8 @@ svn_cl__cat(apr_getopt_t *os,
   apr_pool_t *subpool = svn_pool_create(pool);
 
   SVN_ERR(svn_cl__args_to_target_array_print_reserved(&targets, os,
-                                                      opt_state->targets, 
-                                                      pool));
+                                                      opt_state->targets,
+                                                      ctx, pool));
 
   /* Cat cannot operate on an implicit '.' so a filename is required */
   if (! targets->nelts)
